/*  BEGIN software license
 *
 *  msXpertSuite - mass spectrometry software suite
 *  -----------------------------------------------
 *  Copyright(C) 2009, 2017 Filippo Rusconi
 *
 *  http://www.msxpertsuite.org
 *
 *  This file is part of the msXpertSuite project.
 *
 *  The msXpertSuite project is the successor of the massXpert project. This
 *  project now includes various independent modules:
 *  
 *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
 *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * END software license
 */



/////////////////////// Qt includes
#include <QDebug>

/////////////////////// Local includes
#include "MassPeak.hpp"


namespace msXpSlibmass
{

	//! Constructor.
	/*! 

		Initialize \c m_mz and \c m_i to 0.

*/
	MassPeak::MassPeak()
	{
	}


	//! Constructor.
	/*!

		Initialize the member data with the parameters.

		\param mz copied to \c m_mz
		\param i copied to \c m_i

		*/
	MassPeak::MassPeak(double mz, double i)
	{
		m_mz = mz;
		m_i = i;
	}


	//! Constructor.
	/*!

		Constructs \c this to be identical to \p massPeak.

		*/
	MassPeak::MassPeak(const MassPeak &massPeak)
	{
		m_mz = massPeak.m_mz;
		m_i = massPeak.m_i;
	}


	//! Destructor.
	MassPeak::~MassPeak()
	{
	}


	//! Return the \c m_mz value.
	/*!

		\return \c m_mz.
	
		*/
	double
		MassPeak::mz() const
		{
			return m_mz;
		}


	//! Return the \c m_i value.
	/*!

		\return \c m_i.
	
		*/
	double 
		MassPeak::i() const
		{
			return m_i;
		}


	//! Return a <em> non-const reference</em> to \c m_i.
	/*!

		\return a <em> non-const reference</em> to \c m_i.
	
		*/
	double &
		MassPeak::ri()
		{
			return m_i;
		}



} // namespace msXpSlibmass
