/*  BEGIN software license
 *
 *  msXpertSuite - mass spectrometry software suite
 *  -----------------------------------------------
 *  Copyright(C) 2009, 2017 Filippo Rusconi
 *
 *  http://www.msxpertsuite.org
 *
 *  This file is part of the msXpertSuite project.
 *
 *  The msXpertSuite project is the successor of the massXpert project. This
 *  project now includes various independent modules:
 *  
 *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
 *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * END software license
*/



#ifndef COLOR_MAP_PLOT_WIDGET_HPP
#define COLOR_MAP_PLOT_WIDGET_HPP

/////////////////////// Qt includes
#include <QObject>
#include <QString>
#include <QWidget>
#include <QColor>


/////////////////////// QCustomPlot
#include "AbstractPlotWidget.hpp"
#include "AbstractMultiPlotWnd.hpp"
#include "MassSpecDataSet.hpp"
#include "History.hpp"

///////////////////////////// Local includes


namespace msXpSmineXpert
{


  class ColorMapPlotWidget : public AbstractPlotWidget
  {
    Q_OBJECT

    public:
    IntegrationType arbitraryIntegrationType();

    bool setupWidget();

    // Construction/Desctruction
    ColorMapPlotWidget(QWidget *parent, AbstractMultiPlotWnd *parentWnd,
                       const QString &name, const QString &desc,
                       const QString &  fileName        = QString(),
                       MassSpecDataSet *massSpecDataSet = nullptr);

    ~ColorMapPlotWidget();

    void mouseReleaseHandledEvent(QMouseEvent *       event,
                                  AbstractPlotWidget *emitter);
    void keyReleaseEvent(QKeyEvent *event);

			Q_INVOKABLE void integrateToMz(double xAxisLower, double xAxisUpper,
					double yAxisLower, double yAxisUpper);
			Q_INVOKABLE void integrateToDt(double xAxisLower, double xAxisUpper,
					double yAxisLower, double yAxisUpper);
			Q_INVOKABLE void integrateToRt(double xAxisLower, double xAxisUpper,
					double yAxisLower, double yAxisUpper);
			Q_INVOKABLE double integrateToTicIntensity(double xAxisLower, double xAxisUpper,
					double yAxisLower, double yAxisUpper);


    signals:
    void recordAnalysisStanza(QString stanza, const QColor &color = QColor());

    void newTicChromatogramByDtAndMz(MassSpecDataSet *massSpecDataSet,
                                  IntegrationType  integrationType,
                                  const History &history, QColor color);
		
    void newMassSpectrumByDtAndMz(MassSpecDataSet *massSpecDataSet,
                                  IntegrationType  integrationType,
                                  const History &history, QColor color);

    void newDriftSpectrumByDtAndMz(MassSpecDataSet *massSpecDataSet,
                                   IntegrationType  integrationType,
                                   const History &history, QColor color);
  };


} // namespace msXpSmineXpert


#endif // COLOR_MAP_PLOT_WIDGET_HPP
