/*  BEGIN software license
 *
 *  msXpertSuite - mass spectrometry software suite
 *  -----------------------------------------------
 *  Copyright(C) 2009, 2017 Filippo Rusconi
 *
 *  http://www.msxpertsuite.org
 *
 *  This file is part of the msXpertSuite project.
 *
 *  The msXpertSuite project is the successor of the massXpert project. This
 *  project now includes various independent modules:
 *  
 *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
 *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * END software license
*/



/////////////////////// Qt includes


/////////////////////// Local includes
#include "DataPlotWidgetRelation.hpp"


namespace msXpSmineXpert
{

  DataPlotWidgetRelation::DataPlotWidgetRelation()
  {
  }

  DataPlotWidgetRelation::~DataPlotWidgetRelation()
  {
  }

  QString
  DataPlotWidgetRelation::asText()
  {
    QString text;

    text += QString("Relation: 0x%2 --")
              .arg((quintptr)this, QT_POINTER_SIZE * 2, 16, QChar('0'));

    text += QString("%1: 0x%2 --")
              .arg("mp_dataSet")
              .arg((quintptr)mp_dataSet, QT_POINTER_SIZE * 2, 16, QChar('0'));

    text += QString("%1: 0x%2 --")
              .arg("mp_originPlotWidget")
              .arg((quintptr)mp_originPlotWidget, QT_POINTER_SIZE * 2, 16,
                   QChar('0'));

    text += QString("%1: 0x%2 --")
              .arg("mp_targetPlotWidget")
              .arg((quintptr)mp_targetPlotWidget, QT_POINTER_SIZE * 2, 16,
                   QChar('0'));

    text += QString("%1: 0x%2 --")
              .arg("mp_openSpectraListWidgetItem")
              .arg((quintptr)mp_openSpectraListWidgetItem, QT_POINTER_SIZE * 2,
                   16, QChar('0'));

    return text;
  }

  DataPlotWidgetRelationer::DataPlotWidgetRelationer()
  {
  }

  DataPlotWidgetRelationer::~DataPlotWidgetRelationer()
  {
    // Free all the relations that were allocated on the heap.
    for(int iter = 0; iter < m_list.size(); ++iter)
      delete m_list.at(iter);
  }

  QString
  DataPlotWidgetRelationer::asText()
  {
    QString text;

    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        text += m_list.at(iter)->asText();
        text += "\n";
      }

    return text;
  }

  void
  DataPlotWidgetRelationer::newRelation(MassSpecDataSet *   dataSet,
                                        AbstractPlotWidget *orig,
                                        AbstractPlotWidget *target,
                                        QListWidgetItem *   widgetItem)
  {
    // Only two data members cannot be nullptr when calling this function:
    if(dataSet == nullptr || orig == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    DataPlotWidgetRelation *p_relation = new DataPlotWidgetRelation;

    p_relation->mp_dataSet                   = dataSet;
    p_relation->mp_originPlotWidget          = orig;
    p_relation->mp_targetPlotWidget          = target;
    p_relation->mp_openSpectraListWidgetItem = widgetItem;

    m_list.append(p_relation);
  }


#if 0
    // Returns true if the relation was updated and false if a new one was added.

    bool
    DataPlotWidgetRelationer::updateOrAddRelation(MassSpecDataSet *dataSet,
            AbstractPlotWidget *origin,
            AbstractPlotWidget *target,
            QListWidgetItem *widgetItem) {
        if (dataSet == nullptr || origin == nullptr || target == nullptr)
            qFatal("Fatal error at %s@%d. Program aborted.",
                __FILE__, __LINE__);

        for (int iter = 0; iter < m_list.size(); ++iter) {
            DataPlotWidgetRelation *p_relation = m_list.at(iter);

            // We only update the relation if the target plot widget pointer is
            // nullptr ! This is it: we had created a new relation, typically when
            // opening a new mass spec file. But the user had not had the
            // opportunity to make an integration operation, so the target is
            // nullptr. But, eventually, such an integration would occur. Then, yes,
            // we update. Now, after the first relation object has been updated,
            // there won't be any relation object that has its target pointer with
            // value nullptr, which means we have to actually duplicate the found
            // object and fill-in the target value.
            if (p_relation->mp_dataSet == dataSet &&
                    p_relation->mp_originPlotWidget == origin &&
                    p_relation->mp_targetPlotWidget == nullptr) {
                if (widgetItem != nullptr) {
                    if (p_relation->mp_openSpectraListWidgetItem != widgetItem)
                        qFatal("Fatal error at %s@%d. Program aborted.",
                            __FILE__, __LINE__);
                }

                p_relation->mp_targetPlotWidget = target;
                return true;
            }
        }

        // There was no matching object with a nullptr target value. Then we
        // need to create one.

        // First, we need to get a pointer to the QListWidgetItem.
        QListWidgetItem *item = listWidgetItem(origin);

        newRelation(dataSet, origin, target, item);

        return false;
    }
#endif

  void
  DataPlotWidgetRelationer::removeRelation(DataPlotWidgetRelation *relation)
  {
    if(relation == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    if(!m_list.removeOne(relation))
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
  }

  void
  DataPlotWidgetRelationer::clearRelation(DataPlotWidgetRelation *relation)
  {
    if(relation == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    // Only actually delete the relation if it was indeed found in the list
    // and removed from it.
    if(m_list.removeOne(relation))
      {
        delete relation;
        relation = nullptr;
      }
    else
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
  }

  void
  DataPlotWidgetRelationer::getRelationsByOrigin(
    AbstractPlotWidget *origin, QList<DataPlotWidgetRelation *> *list)
  {
    if(origin == nullptr || list == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        if(p_relation->mp_originPlotWidget == origin)
          list->append(p_relation);
      }
  }

  void
  DataPlotWidgetRelationer::getRelationsByOriginRecursive(
    AbstractPlotWidget *origin, QList<DataPlotWidgetRelation *> *list) const
  {
    if(origin == nullptr || list == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    QString msg =
      QString("Entering getRelationsByOriginRecursive with origin 0x%1\n")
        .arg((quintptr)origin, QT_POINTER_SIZE * 2, 16, QChar('0'));
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wformat-security"
    printf("%s", msg.toLatin1().data());
#pragma clang diagnostic pop


    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        printf("iter: %d relation: %s\n", iter,
               p_relation->asText().toLatin1().data());

        if(p_relation->mp_originPlotWidget == origin)
          {
            printf("\tmatching origin %s\n",
                   p_relation->asText().toLatin1().data());

            list->append(p_relation);

            // Because we want recursively all the targets, we can now consider
            // that this matching origin plot widget's target widget is now an
            // origin plot widget:
            if(p_relation->mp_targetPlotWidget != nullptr)
              getRelationsByOriginRecursive(p_relation->mp_targetPlotWidget,
                                            list);
          }
      }
  }

  void
  DataPlotWidgetRelationer::getRelationsByTarget(
    AbstractPlotWidget *target, QList<DataPlotWidgetRelation *> *list)
  {
    if(target == nullptr || list == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        if(p_relation->mp_targetPlotWidget == target)
          list->append(p_relation);
      }
  }

  void
  DataPlotWidgetRelationer::getRelationsByData(
    MassSpecDataSet *dataSet, QList<DataPlotWidgetRelation *> *list)
  {
    if(dataSet == nullptr || list == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        if(p_relation->mp_dataSet == dataSet)
          list->append(p_relation);
      }
  }

  void
  DataPlotWidgetRelationer::getRelationsByOpenSpectraListWidgetItem(
    QListWidgetItem *item, QList<DataPlotWidgetRelation *> *list)
  {
    if(item == nullptr || list == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        if(p_relation->mp_openSpectraListWidgetItem == item)
          list->append(p_relation);
      }
  }

  AbstractPlotWidget *
  DataPlotWidgetRelationer::origin(AbstractPlotWidget *target)
  {
    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        if(p_relation->mp_targetPlotWidget == target)
          return p_relation->mp_originPlotWidget;
      }

    return nullptr;
  }

  AbstractPlotWidget *
  DataPlotWidgetRelationer::origin(MassSpecDataSet *dataSet)
  {
    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        if(p_relation->mp_dataSet == dataSet)
          return p_relation->mp_originPlotWidget;
      }

    return nullptr;
  }

  AbstractPlotWidget *
  DataPlotWidgetRelationer::origin(QListWidgetItem *item)
  {
    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        if(p_relation->mp_openSpectraListWidgetItem == item)
          return p_relation->mp_originPlotWidget;
      }

    return nullptr;
  }

  int
  DataPlotWidgetRelationer::targets(AbstractPlotWidget *         origin,
                                    QList<AbstractPlotWidget *> *list)
  {
    if(origin == nullptr || list == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    int count = 0;

    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        if(p_relation->mp_originPlotWidget == origin)
          {
            // There might be a root plot widget that has a non-nullptr origin
            // but
            // that has a nullptr target, we do not return it.
            if(p_relation->mp_targetPlotWidget != nullptr)
              {
                list->append(p_relation->mp_targetPlotWidget);
                ++count;
              }
          }
      }

    return count;
  }

  int
  DataPlotWidgetRelationer::targetsRecursive(AbstractPlotWidget *origin,
                                             QList<AbstractPlotWidget *> *list)
  {
    if(origin == nullptr || list == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    int count = 0;

    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        if(p_relation->mp_originPlotWidget == origin)
          {
            // There might be a root plot widget that has a non-nullptr origin
            // but
            // that has a nullptr target, we do not return it.
            if(p_relation->mp_targetPlotWidget != nullptr)
              {
                list->append(p_relation->mp_targetPlotWidget);
                ++count;

                // Because we are asked for a recursive search of all the
                // targets of
                // origin, we must now turn this found target into an origin:
                targetsRecursive(p_relation->mp_targetPlotWidget, list);
              }
          }
      }

    return count;
  }

  bool
  DataPlotWidgetRelationer::isLeafPlotWidget(AbstractPlotWidget *    widget,
                                             DataPlotWidgetRelation *relation)
  {
    // If the widget is the last of a branch, that is, it has no target
    // widget, in the sense that precisely it is only a target plot widget,
    // then we should return true.

    QList<DataPlotWidgetRelation *> relationList;
    getRelationsByOrigin(widget, &relationList);

    if(relationList.size())
      {
        // qDebug() << __FILE__ << __LINE__
        // << "The plot widget is an origin plot widget, thus not leaf.
        // Returning false.";

        return false;
      }

    // If the widget is not part of any relation as an origin, then it must be
    // part of exactly one relation as a target plot widget.
    getRelationsByTarget(widget, &relationList);

    if(relationList.size() == 1)
      {
        if(relation != nullptr)
          relation = relationList.first();

        return true;
      }
    else
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    return false;
  }

  QListWidgetItem *
  DataPlotWidgetRelationer::listWidgetItem(AbstractPlotWidget *origin)
  {
    if(origin == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    for(int iter = 0; iter < m_list.size(); ++iter)
      {
        DataPlotWidgetRelation *p_relation = m_list.at(iter);

        if(p_relation->mp_originPlotWidget == origin)
          return p_relation->mp_openSpectraListWidgetItem;
      }

    return nullptr;
  }

  int
  DataPlotWidgetRelationer::removeOrigin(AbstractPlotWidget *origin)
  {
    if(origin == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    // When we remove an origin, that is, a TicChromPlotwidget for example, we
    // may have many such items because there are as many such items as there
    // were plot widgets created by integrating in the TIC chrom plot widget,
    // either once or more to MassSpec or one or more to DriftSpec, for
    // example. We need to remove all the items in m_list that math origin.

    // Get a list of relations for which m_originPlotWidget equals the origin
    // parameter.
    QList<DataPlotWidgetRelation *> list;
    getRelationsByOrigin(origin, &list);

    int count = 0;

    // Now iterate in that list and for each matching relation in the member
    // m_list, remove it from there.
    for(int iter = 0; iter < list.size(); ++iter)
      {
        DataPlotWidgetRelation *curRelation = list.at(iter);

        for(int jter = 0; jter < m_list.size(); ++jter)
          {
            if(curRelation == m_list.at(jter))
              {
                delete m_list.takeAt(jter);
                --jter;
                ++count;
              }
          }
      }

    return count;
  }


} // namespace msXpSmineXpert

