/*  BEGIN software license
 *
 *  msXpertSuite - mass spectrometry software suite
 *  -----------------------------------------------
 *  Copyright(C) 2009, 2017 Filippo Rusconi
 *
 *  http://www.msxpertsuite.org
 *
 *  This file is part of the msXpertSuite project.
 *
 *  The msXpertSuite project is the successor of the massXpert project. This
 *  project now includes various independent modules:
 *  
 *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
 *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * END software license
*/



#ifndef DATA_PLOT_WIDGET_RELATION_HPP
#define DATA_PLOT_WIDGET_RELATION_HPP


/////////////////////// Qt includes
#include <QListWidgetItem>


/////////////////////// Local includes
#include "AbstractPlotWidget.hpp"
#include "MassSpecDataSet.hpp"

namespace msXpSmineXpert
{

	class DataPlotWidgetRelation : public QObject
	{
		Q_OBJECT

		public:
			MassSpecDataSet *mp_dataSet = nullptr;
			AbstractPlotWidget *mp_originPlotWidget = nullptr;
			AbstractPlotWidget *mp_targetPlotWidget = nullptr;
			QListWidgetItem * mp_openSpectraListWidgetItem = nullptr;

			DataPlotWidgetRelation();
			~DataPlotWidgetRelation();

			QString asText();
	};


	class DataPlotWidgetRelationer : public QObject
	{
		Q_OBJECT

			friend class DataPlotWidgetRelation;

		public:
		QList<DataPlotWidgetRelation *> m_list;

		DataPlotWidgetRelationer();
		~DataPlotWidgetRelationer();

		Q_INVOKABLE QString asText();

		void newRelation(MassSpecDataSet *dataSet, AbstractPlotWidget *origin,
				AbstractPlotWidget *target, QListWidgetItem *widgetItem);

#if 0
		bool updateOrAddRelation(MassSpecDataSet *dataSet,
				AbstractPlotWidget *origin,
				AbstractPlotWidget *target,
				QListWidgetItem *widgetItem);
#endif

		void removeRelation(DataPlotWidgetRelation *relation);
		void clearRelation(DataPlotWidgetRelation *relation);

		void getRelationsByOrigin(AbstractPlotWidget * origin,
				QList<DataPlotWidgetRelation *> *list);
		void getRelationsByOriginRecursive(AbstractPlotWidget * origin,
				QList<DataPlotWidgetRelation *> *list) const;

		void getRelationsByTarget(AbstractPlotWidget * target,
				QList<DataPlotWidgetRelation *> *list);
		void getRelationsByData(MassSpecDataSet * dataSet,
				QList<DataPlotWidgetRelation *> *list);
		void getRelationsByOpenSpectraListWidgetItem(
				QListWidgetItem *item, QList<DataPlotWidgetRelation *> *list);

		AbstractPlotWidget *origin(AbstractPlotWidget *target);
		AbstractPlotWidget *origin(MassSpecDataSet *dataSet);
		AbstractPlotWidget *origin(QListWidgetItem *item);

		int targets(AbstractPlotWidget *origin, QList<AbstractPlotWidget *> *list);
		int targetsRecursive(AbstractPlotWidget * origin,
				QList<AbstractPlotWidget *> *list);

		bool isLeafPlotWidget(AbstractPlotWidget * widget,
				DataPlotWidgetRelation *relation = nullptr);

		QListWidgetItem *listWidgetItem(AbstractPlotWidget *origin);

		int removeOrigin(AbstractPlotWidget *origin);
	};

} // namespace msXpSmineXpert


#endif /* DATA_PLOT_WIDGET_RELATION_HPP */
