/*  BEGIN software license
 *
 *  msXpertSuite - mass spectrometry software suite
 *  -----------------------------------------------
 *  Copyright(C) 2009, 2017 Filippo Rusconi
 *
 *  http://www.msxpertsuite.org
 *
 *  This file is part of the msXpertSuite project.
 *
 *  The msXpertSuite project is the successor of the massXpert project. This
 *  project now includes various independent modules:
 *  
 *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
 *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * END software license
*/



#ifndef TIC_CHROM_WND_HPP
#define TIC_CHROM_WND_HPP

/////////////////////// Qt includes
#include <QObject>
#include <QString>
#include <QWidget>


/////////////////////// Local includes
#include "AbstractMultiPlotWnd.hpp"
#include "TicChromPlotWidget.hpp"
#include "AnalysisPreferences.hpp"
#include "MassSpecDataFileLoaderSqlite3.hpp"
#include "MassSpecDataSet.hpp"


namespace msXpSmineXpert
{

	class TicChromWnd : public AbstractMultiPlotWnd
	{
		Q_OBJECT

		public:
			QFile *mp_analysisFile = nullptr;
			// The analysis preferences are set from the MainWindow when the user sets
			// them with the dedicated dialog window.
			AnalysisPreferences *mp_analysisPreferences  = nullptr;
			TicChromPlotWidget * mp_multiGraphPlotWidget = nullptr;

			static int widgetCounter;

			TicChromWnd(QWidget *parent);
			~TicChromWnd();

			void initialize();
			void initializePlotRegion();

			Q_INVOKABLE void hide(){QMainWindow::hide();};
			Q_INVOKABLE void show(){QMainWindow::show();};

			TicChromPlotWidget *
				addPlotWidget(QVector<double> keyVector, QVector<double> valVector,
						const QString &  desc            = QString(),
						MassSpecDataSet *massSpecDataSet = nullptr);

			AbstractPlotWidget *initialTic(MassSpecDataSet *massSpecDataSet);
			void fakeData(QVector<double> *keyVector, QVector<double> *valVector);

			void setAnalysisPreferences(AnalysisPreferences *preferences);
			void setAnalysisFile(QFile *analysisFile);

			void newTicChromatogramByMz(MassSpecDataSet *massSpecDataSet,
					IntegrationType  integrationType,
					History history, QColor color);

			void newTicChromatogramByDt(MassSpecDataSet *massSpecDataSet,
					IntegrationType  integrationType,
					History history, QColor color);

			void ticIntensity(MassSpecDataSet *massSpecDataSet,
					double *intensity, IntegrationType integrationType, History history);

			void newStreamedTicChromatogram(MassSpecDataSet *massSpecDataSet,
					IntegrationType  integrationType,
					QVector<double> *keyVector,
					QVector<double> *valVector, History history,
					QColor color);

			void streamedTicIntensity(MassSpecDataSet *massSpecDataSet,
					double *intensity, IntegrationType integrationType, History history);

			Q_INVOKABLE void clearPlots();

			// We do not want to hide the parent class function here.
			using AbstractMultiPlotWnd::toggleMultiGraphGraph;
			void toggleMultiGraphGraph();

			void recordAnalysisStanza(const QString &stanza,
					const QColor & color = QColor());
	};


} // namespace msXpSmineXpert


#endif /* TIC_CHROM_WND_HPP */
